package net.wanji.utc.config.security;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.security.oauth2.OAuth2ClientProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.*;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.client.resource.OAuth2ProtectedResourceDetails;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.provider.endpoint.TokenEndpoint;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.*;

@Configuration
public class OAuth2AccessTokenRestTemplate {
    private final String GRANT_TYPE_PASSWORD = "password";
    private final String GRANT_TYPE_CLIENT_CREDENTIALS = "client_credentials";
    private final String USERNAME = "username";
    private final String PASSWORD = "password";
    private final String GRANT_TYPE = "grant_type";
    private final String GRANT_SCOPE = "scope";
    @Autowired
    OAuth2ClientProperties oAuth2ClientProperties;
    @Autowired
    OAuth2ProtectedResourceDetails oAuth2ProtectedResourceDetails;
    private RestTemplate restTemplate;
    @Autowired
    @Lazy
    TokenEndpoint tokenEndpoint;

    public OAuth2AccessTokenRestTemplate() {
    }

    private RestTemplate getRestTemplate() {
        if (null == this.restTemplate) {
            this.restTemplate = new RestTemplate();
        }

        return this.restTemplate;
    }

    public OAuth2AccessToken getTokenWithPasswordGrantType(String username, String password, Map<String, String> ext) throws Exception {
        return this.getToken((String)null, (String)null, username, password, "password", ext);
    }

    public OAuth2AccessToken getTokenWithPasswordGrantType(String username, String password) throws Exception {
        return this.getTokenWithPasswordGrantType(username, password, (Map)null);
    }

    public OAuth2AccessToken getTokenWithPasswordGrantType(String clientId, String clientSecret, String username, String password, Map<String, String> ext) throws Exception {
        return this.getToken(clientId, clientSecret, username, password, "password", ext);
    }

    public OAuth2AccessToken getTokenWithPasswordGrantType(String clientId, String clientSecret, String username, String password) throws Exception {
        return this.getTokenWithPasswordGrantType(clientId, clientSecret, username, password, (Map)null);
    }

    public OAuth2AccessToken getTokenWithClientCredentialsGrantType(Map<String, String> ext) throws Exception {
        return this.getToken((String)null, (String)null, (String)null, (String)null, "client_credentials", ext);
    }

    public OAuth2AccessToken getTokenWithClientCredentialsGrantType() throws Exception {
        return this.getTokenWithClientCredentialsGrantType((Map)null);
    }

    public OAuth2AccessToken getTokenWithClientCredentialsGrantType(String clientId, String clientSecret, Map<String, String> ext) throws Exception {
        return this.getToken(clientId, clientSecret, (String)null, (String)null, "client_credentials", ext);
    }

    public OAuth2AccessToken getTokenWithClientCredentialsGrantType(String clientId, String clientSecret) throws Exception {
        return this.getTokenWithClientCredentialsGrantType(clientId, clientSecret, (Map)null);
    }

    public OAuth2AccessToken getTokenWithAuthorizationCodeGrantType(Map<String, String> ext) throws Exception {
        return null;
    }

    public OAuth2AccessToken getTokenWithAuthorizationCodeGrantType() throws Exception {
        return this.getTokenWithAuthorizationCodeGrantType((Map)null);
    }

    private OAuth2AccessToken getToken(String clientId, String clientSecret, String username, String password, String grantType, Map<String, String> ext) throws Exception {
        long start = System.currentTimeMillis();
        DefaultOAuth2AccessToken accessToken = null;
        ResponseEntity response = null;

        try {
            System.out.println("========1.1=======OAuth2AccessTokenRestTemplate======getToken=========" + (System.currentTimeMillis() - start));
            UserDetails user = User.builder().username(StringUtils.isBlank(clientId) ? this.oAuth2ClientProperties.getClientId() : clientId).password(StringUtils.isBlank(clientSecret) ? this.oAuth2ClientProperties.getClientSecret() : clientSecret).roles(new String[]{"admin"}).build();
            response = this.tokenEndpoint.postAccessToken(new UsernamePasswordAuthenticationToken(user, password, user.getAuthorities()), new HashMap<String, String>() {
                {
                    this.put("username", username);
                    this.put("password", password);
                    this.put("grant_type", grantType);
                    this.put("scope", String.join(",", OAuth2AccessTokenRestTemplate.this.oAuth2ProtectedResourceDetails.getScope()));
                    this.putAll(ext);
                }
            });
            System.out.println("========1.2=======OAuth2AccessTokenRestTemplate======postAccessToken=========" + (System.currentTimeMillis() - start));
            accessToken = (DefaultOAuth2AccessToken)JSONObject.toJavaObject((JSON)JSONObject.toJSON(response.getBody()), DefaultOAuth2AccessToken.class);
            System.out.println("========1.3=======OAuth2AccessTokenRestTemplate======toJavaObject=========" + (System.currentTimeMillis() - start));
        } catch (Exception var12) {
            throw new Exception(var12.getMessage());
        }

        if (!Objects.isNull(accessToken) && !StringUtils.isBlank(accessToken.getValue())) {
            return accessToken;
        } else {
            throw new Exception((String)((Map)response.getBody()).get("message"));
        }
    }

    private OAuth2AccessToken getTokenByHttp(String clientId, String clientSecret, String username, String password, String grantType, Map<String, String> ext) throws Exception {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        httpHeaders.setBasicAuth(StringUtils.isBlank(clientId) ? this.oAuth2ClientProperties.getClientId() : clientId, StringUtils.isBlank(clientSecret) ? this.oAuth2ClientProperties.getClientSecret() : clientSecret);
        MultiValueMap<String, String> map = new LinkedMultiValueMap();
        if (StringUtils.isNotEmpty(username)) {
            map.put("username", Collections.singletonList(username));
        }

        if (StringUtils.isNotEmpty(password)) {
            map.put("password", Collections.singletonList(password));
        }

        map.put("grant_type", Collections.singletonList(grantType));
        map.put("scope", this.oAuth2ProtectedResourceDetails.getScope());
        if (null != ext && !ext.isEmpty()) {
            ext.forEach((k, v) -> {
                List var10000 = (List)map.put(k, Collections.singletonList(v));
            });
        }

        HttpEntity httpEntity = new HttpEntity(map, httpHeaders);
        DefaultOAuth2AccessToken accessToken = null;
        ResponseEntity response = null;

        try {
            response = this.getRestTemplate().exchange(this.oAuth2ProtectedResourceDetails.getAccessTokenUri(), HttpMethod.POST, httpEntity, Object.class, new Object[0]);
            accessToken = (DefaultOAuth2AccessToken)JSONObject.toJavaObject((JSON)JSONObject.toJSON(response.getBody()), DefaultOAuth2AccessToken.class);
        } catch (Exception var13) {
            throw new Exception(var13.getMessage());
        }

        if (!Objects.isNull(accessToken) && !StringUtils.isBlank(accessToken.getValue())) {
            return accessToken;
        } else {
            throw new Exception((String)((Map)response.getBody()).get("message"));
        }
    }
}
